#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "allegro.h"
#include "2xsai.h"

volatile int chrono = 0;

void the_timer(void) {
	chrono++;
} END_OF_FUNCTION(the_timer);



int main(int argc, char *argv[])
{
   BITMAP *dstBitmap, *loadBitmap;
   PALETTE pal;
   char *src_filename = NULL;
   char *dst_filename = NULL;
   uint8 *deltaPtr = NULL;
   int dstwidth = 640;
   int dstheight = 480;
	int arg_error = 0;
	int depth = 16;
	int engine = 1;
	int i;
	float fps = 0.0;
 

   allegro_message("\n2xSaI Benchmarking Utility v0.1" \
   		"\n(c) 2001, Robert J Ohannessian"
   		"\nThis program is free for non-commercial use." \
   		"\nSee 2xSai.txt for more info\n");

	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-?") || !strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
			arg_error = 1;
		}
		if (!strcmp(argv[i], "-bpp")) {
			if (i + 1 < argc)
				depth = atoi(argv[i + 1]);
			if ((i + 1 >= argc) || ((depth != 16) && (depth != 32) && (depth != 15))) {
				arg_error = 1;
				break;
			}
			else
				i++;
		}
		if (!strcmp(argv[i], "-e")) {
			if (i + 1 < argc)
				engine = atoi(argv[i + 1]);
			if ((i + 1 >= argc) || (engine < 1) || (engine > 2)) {
				arg_error = 1;
				break;
			}
			else
				i++;
		}
	}
		
	if (argc < 3 || arg_error) {
		allegro_message("Usage: 2xsai source dest [-bpp 15|16|32] [-e 1|2]\n");
		return 1;
	}

   src_filename = argv[1];  //first parameter is source file
   dst_filename = argv[2];  //second parameter is destination file


    //initialize allegro
   allegro_init();

   //do not delete this line !!! Allegro bitmap conversion routines won't work.
   set_color_depth(depth);
   Init_2xSaI(depth);


   //load the bitmap from file
   loadBitmap = load_bitmap(src_filename, pal);
   if (!loadBitmap)
   {
      allegro_message("\nError reading source file");
      return 2;
   }


   dstwidth = 2*loadBitmap->w;
   dstheight = 2*loadBitmap->h;


   //memory for destination bitmap. Dimensions are 2 times bigger than source bitmap
   dstBitmap = create_bitmap(dstwidth, dstheight);
   
   LOCK_FUNCTION(the_timer);
   LOCK_VARIABLE(chrono);
   install_int(the_timer, 1);
   
	{
	int chrono2;   
	int fcount = 0;
	
	while (chrono < 10000) {
		if (engine == 1)
			Super2xSaI(loadBitmap, dstBitmap, 0, 0, 0, 0, loadBitmap->w, loadBitmap->h);
		else if (engine == 2)
			SuperEagle(loadBitmap, dstBitmap, 0, 0, 0, 0, loadBitmap->w, loadBitmap->h);
		fcount++;
    }
    chrono2 = chrono;
    
    fps = fcount * 1000.0 / chrono2;
    
    }
    

   //save the bitmap
   if (save_bitmap(dst_filename, dstBitmap, pal) != 0)
      allegro_message("\nError writing file");
   else
   {
      allegro_message("\nFile successfully written, fps: %.2f", fps);
   }

   destroy_bitmap(loadBitmap);
   destroy_bitmap(dstBitmap);

   return 0;
} END_OF_MAIN();
