
with Allegro;                           use Allegro;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Version;

package body Applications.Shell is

    overriding
    procedure Close( this : access Shell_Application ) is
    begin
        pragma Debug( Dbg( "Shutting down Allegro", D_INIT, INFO ) );
        Allegro_Exit;
        pragma Debug( Dbg( "Application closed", D_INIT, INFO ) );

        Application(this.all).Close;
    end Close;

    ----------------------------------------------------------------------------

    overriding
    function Init( this : access Shell_Application ) return Boolean is
    begin
        return this.Init( enableAllegro => False );
    end Init;

    ----------------------------------------------------------------------------

    not overriding
    function Init( this          : access Shell_Application;
                   enableAllegro : Boolean ) return Boolean is
    begin
        if not Application(this.all).Init then
            return False;
        end if;

        if (enableAllegro and then Allegro_Initialize /= 0) or else
           (not enableAllegro and then Allegro_Initialize_Cmdline /= 0)
        then
            Allegro_Message( "Error initializing Allegro" );
            pragma Debug( Dbg( "Application.Init Error: Allegro_Initialize: " &
                               Allegro_Error, D_INIT, ERROR ) );
            raise INIT_EXCEPTION;
        end if;

        return True;
    exception
        when e : others =>
            pragma Debug( Dbg( "Shell_Application.Init Exception: ...", D_INIT, ERROR ) );
            pragma Debug( Dbg( e, D_INIT, ERROR ) );

            this.Close;
            return False;
    end Init;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Announce( this : access Shell_Application ) return String is
    begin
        return this.Get_Name & " v" & Version.Release & " - " &
               this.Get_Company & " " &
               "Copyright (C) " & Version.Copyright_Year;
    end Get_Announce;

end Applications.Shell;
