
package body Directions is

    function "+"( l : Direction_Type; r : Cardinal_Direction ) return Direction_Type is
        result : Direction_Type := l;
    begin
        case r is
            when Left  => result.x := -1;
            when Right => result.x :=  1;
            when Up    => result.y := -1;
            when Down  => result.y := 1;
        end case;
        return result;
    end "+";
    pragma Inline( "+" );

    ----------------------------------------------------------------------------

    function "-"( l : Direction_Type; r : Cardinal_Direction ) return Direction_Type is
        result : Direction_Type := l;
    begin
        case r is
            when Left =>
                if result.x < 0 and then result.y /= 0 then
                    result.x := 0;
                end if;
            when Right =>
                if result.x > 0 and then result.y /= 0 then
                    result.x := 0;
                end if;
            when Up =>
                if result.y < 0 and then result.x /= 0 then
                    result.y := 0;
                end if;
            when Down =>
                if result.y > 0 and then result.x /= 0 then
                    result.y := 0;
                end if;
        end case;
        return result;
    end "-";
    pragma Inline( "-" );

    ----------------------------------------------------------------------------

    function "and"( l : Direction_Type; r : Cardinal_Direction ) return Boolean is
    begin
        case r is
            when Left  => return l.x < 0;
            when Right => return l.x > 0;
            when Up    => return l.y < 0;
            when Down  => return l.y > 0;
        end case;
    end "and";
    pragma Inline( "and" );

    ----------------------------------------------------------------------------

    function To_D8( dir : Direction_Type ) return Direction_8 is
    begin
        case dir.y is
            when -1 =>
                case dir.x is
                    when -1 => return D8_Up_Left;
                    when  0 => return D8_Up;
                    when  1 => return D8_Up_Right;
                end case;
            when  0 =>
                case dir.x is
                    when -1 => return D8_Left;
                    when  0 => raise Constraint_Error with "Invalid direction";
                    when  1 => return D8_Right;
                end case;
            when  1 =>
                case dir.x is
                    when -1 => return D8_Down_Left;
                    when  0 => return D8_Down;
                    when  1 => return D8_Down_Right;
                end case;
        end case;
    end To_D8;
    pragma Inline( To_D8 );

    ----------------------------------------------------------------------------

    function To_D8( dir : Cardinal_Direction ) return Direction_8 is
    begin
        case dir is
            when Left  => return D8_Left;
            when Right => return D8_Right;
            when Up    => return D8_Up;
            when Down  => return D8_Down;
        end case;
    end To_D8;
    pragma Inline( To_D8 );

    ----------------------------------------------------------------------------

    function To_X( dir : Direction_Type ) return Direction_8 is
    begin
        if dir.x > 0 then
            return D8_Right;
        end if;
        return D8_Left;
    end To_X;
    pragma Inline( To_X );

    ----------------------------------------------------------------------------

    function To_Y( dir : Direction_Type ) return Direction_8 is
    begin
        if dir.y > 0 then
            return D8_Down;
        end if;
        return D8_Up;
    end To_Y;
    pragma Inline( To_Y );

    ----------------------------------------------------------------------------

    function To_Y( dir : Direction_Type ) return Cardinal_Direction is
    begin
        if dir.y > 0 then
            return Down;
        end if;
        return Up;
    end To_Y;
    pragma Inline( To_Y );

    ----------------------------------------------------------------------------

    function "not"( db : Direction_Booleans ) return Boolean is
    begin
        for i in db'Range loop
            if db(i) then
                return False;
            end if;
        end loop;
        return True;
    end "not";
    pragma Inline( "not" );

    ----------------------------------------------------------------------------

    function Opposite( dir : Cardinal_Direction ) return Cardinal_Direction is
    begin
        case dir is
            when Left  => return Right;
            when Right => return Left;
            when Up    => return Down;
            when Down  => return Up;
        end case;
    end Opposite;
    pragma Inline( Opposite );

    ----------------------------------------------------------------------------

    function To_Direction( dir : Cardinal_Direction ) return Direction_Type is
    begin
        case dir is
            when Left  => return Dir_Left;
            when Right => return Dir_Right;
            when Up    => return Dir_Up;
            when Down  => return Dir_Down;
        end case;
    end To_Direction;
    pragma Inline( To_Direction );

    ----------------------------------------------------------------------------

    function To_Direction( dir : Direction_8 ) return Direction_Type is
    begin
        case dir is
            when D8_Left       => return Dir_Left;
            when D8_Right      => return Dir_Right;
            when D8_Up         => return Dir_Up;
            when D8_Down       => return Dir_Down;
            when D8_Up_Left    => return Dir_Up_Left;
            when D8_Up_Right   => return Dir_Up_Right;
            when D8_Down_Left  => return Dir_Down_Left;
            when D8_Down_Right => return Dir_Down_Right;
        end case;
    end To_Direction;
    pragma Inline( To_Direction );

    ----------------------------------------------------------------------------

    function To_String( dir : Direction_Type ) return String is
    begin
        return "[" & Axis_Direction'Image( dir.x ) & ',' &
                     Axis_Direction'Image( dir.y ) & ']';
    end To_String;

end Directions;
