
with Allegro.Drawing;                   use Allegro.Drawing;
with Allegro.Graphics;                  use Allegro.Graphics;
with Allegro.Sprites;                   use Allegro.Sprites;
with Allegro.Transparency;              use Allegro.Transparency;
--with Debugging;                         use Debugging;
with Tanskanen;

package body Drawing_Contexts is

    function Create_Drawing_Context( bmp   : not null A_Bitmap;
                                     offx,
                                     offy  : Integer;
                                     layer : Layer_Type ) return Drawing_Context is
        dc : Drawing_Context;
    begin
        dc.bmp := bmp;
        dc.offx := offx;
        dc.offy := offy;
        dc.layer := layer;
        return dc;
    end Create_Drawing_Context;

    ----------------------------------------------------------------------------

    procedure Blit( dc       : Drawing_Context;
                    source   : A_Bitmap;
                    source_x,
                    source_y,
                    dest_x,
                    dest_y   : Integer;
                    width,
                    height   : Natural ) is
    begin
        if source /= null then
            Blit( source, dc.bmp, source_x, source_y,
                  dest_x - dc.offx, dest_y - dc.offy,
                  width, height );
        end if;
    end Blit;

    ----------------------------------------------------------------------------

    procedure Clear_To_Color( dc : Drawing_Context; color : Color_Type ) is
    begin
        Clear_To_Color( dc.bmp, color );
    end Clear_To_Color;

    ----------------------------------------------------------------------------

    procedure Draw_Alpha_Sprite( dc     : Drawing_Context;
                                 sprite : A_Bitmap;
                                 x, y   : Integer ) is
    begin
        if sprite /= null then
            if Get_Color_Depth = 32 then
                -- Fast ASM blending function
                Tanskanen.AlphaBlend32( sprite, dc.bmp, x - dc.offx, y - dc.offy );

                -- Slow Allegro blending handles all color depths
                --Set_Alpha_Blender;
                --Draw_Trans_Sprite( dc.bmp, sprite, x - dc.offx, y - dc.offy );
                --Solid_Mode;
            else
                -- the transparent portion will show up black, but oh well
                Draw_Sprite( dc.bmp, sprite, x - dc.offx, y - dc.offy );
            end if;
        end if;
    end Draw_Alpha_Sprite;

    ----------------------------------------------------------------------------

    procedure Draw_Sprite( dc     : Drawing_Context;
                           sprite : A_Bitmap;
                           x, y   : Integer ) is
    begin
        if sprite /= null then
            Draw_Sprite( dc.bmp, sprite, x - dc.offx, y - dc.offy );
        end if;
    end Draw_Sprite;

    ----------------------------------------------------------------------------

    function Get_Height( dc : Drawing_Context ) return Natural is
    begin
        return Get_Height( dc.bmp );
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Layer( dc : Drawing_Context ) return Layer_Type is
    begin
        return dc.layer;
    end Get_Layer;

    ----------------------------------------------------------------------------

    function Get_Width( dc : Drawing_Context ) return Natural is
    begin
        return Get_Width( dc.bmp );
    end Get_Width;

    ----------------------------------------------------------------------------

    procedure Line( dc     : Drawing_Context;
                    x1, y1,
                    x2, y2 : Integer;
                    color  : Color_Type ) is
    begin
        if x1 = x2 or else y1 = y2 then
            Fastline( dc.bmp,
                      x1 - dc.offx, y1 - dc.offy,
                      x2 - dc.offx, y2 - dc.offy,
                      color );
        else
            Line( dc.bmp,
                  x1 - dc.offx, y1 - dc.offy,
                  x2 - dc.offx, y2 - dc.offy,
                  color );
        end if;
    end Line;

    ----------------------------------------------------------------------------

    procedure Rect( dc      : Drawing_Context;
                    x1, y1,
                    x2, y2  : Integer;
                    color   : Color_Type;
                    opacity : Natural := 255 ) is
    begin
        if opacity < 255 then
            Set_Trans_Blender( 255, 255, 255, opacity );
            Drawing_Mode( DRAW_MODE_TRANS );
            Rect( dc.bmp, x1 - dc.offx, y1 - dc.offy, x2 - dc.offx, y2 - dc.offy, color );
            Solid_Mode;
        else
            Rect( dc.bmp, x1 - dc.offx, y1 - dc.offy, x2 - dc.offx, y2 - dc.offy, color );
        end if;
    end Rect;

    ----------------------------------------------------------------------------

    procedure Rectfill( dc      : Drawing_Context;
                        x1, y1,
                        x2, y2  : Integer;
                        color   : Color_Type;
                        opacity : Natural := 255 ) is
    begin
        if opacity < 255 then
            Set_Trans_Blender( 255, 255, 255, opacity );
            Drawing_Mode( DRAW_MODE_TRANS );
            Rectfill( dc.bmp, x1 - dc.offx, y1 - dc.offy, x2 - dc.offx, y2 - dc.offy, color );
            Solid_Mode;
        else
            Rectfill( dc.bmp, x1 - dc.offx, y1 - dc.offy, x2 - dc.offx, y2 - dc.offy, color );
        end if;
    end Rectfill;

    ----------------------------------------------------------------------------

    procedure Stretch_Blit( dc            : Drawing_Context;
                            bmp           : A_Bitmap;
                            source_x,
                            source_y      : Integer;
                            source_width,
                            source_height : Positive;
                            dest_x,
                            dest_y        : Integer;
                            dest_width,
                            dest_height   : Positive ) is
    begin
        if bmp /= null then
            Stretch_Blit( bmp, dc.bmp, source_x, source_y,
                          source_width, source_height, dest_x, dest_y,
                          dest_width, dest_height );
        end if;
    end Stretch_Blit;

    ----------------------------------------------------------------------------

    procedure Stretch_Sprite( dc           : Drawing_Context;
                              sprite       : A_Bitmap;
                              x, y         : Integer;
                              w, h         : Positive;
                              proportional : Boolean ) is

        ------------------------------------------------------------------------

        procedure Proportional_Stretch( x, y : Integer ) is
            sw         : constant Positive := Get_Width( sprite );
            sh         : constant Positive := Get_Height( sprite );
            disp_ratio : constant Float := Float(w) / Float(h);
            sprt_ratio : constant Float := Float(sw) / Float(sh);
            new_w,
            new_h      : Natural;
        begin
            if sprt_ratio = disp_ratio then
                -- proportions match
                Allegro.Sprites.Stretch_Sprite( dc.bmp, sprite, x, y, w, h );
            elsif sprt_ratio < disp_ratio then
                -- sprite is taller, adjust width and shift right
                new_w := w * sw / sh; --Integer(Float(w) * sprt_ratio);
                Allegro.Sprites.Stretch_Sprite( dc.bmp, sprite, x + ((w - new_w) / 2), y, new_w, h );
            else
                -- sprite is wider, adjust height
                new_h := h * sh / sw; --Integer(Float(h) / sprt_ratio);
                Allegro.Sprites.Stretch_Sprite( dc.bmp, sprite, x, y + ((h - new_h) / 2), w, new_h );
            end if;
        end Proportional_Stretch;
        pragma Inline_Always( Proportional_Stretch );

        ------------------------------------------------------------------------

    begin
        if sprite /= null then
            if proportional then
                Proportional_Stretch( x - dc.offx, y - dc.offy );
            else
                Allegro.Sprites.Stretch_Sprite( dc.bmp, sprite, x, y, w, h );
            end if;
        end if;
    end Stretch_Sprite;

    ----------------------------------------------------------------------------

    procedure Textout( dc     : Drawing_Context;
                       f      : Font_Type;
                       s      : String;
                       x, y   : Integer;
                       color  : Color_Type;
                       smooth : Boolean ) is
    begin
        if f /= null then
            f.Textout( dc.bmp, s, x - dc.offx, y - dc.offy, color, smooth );
        end if;
    end Textout;

    ----------------------------------------------------------------------------

    procedure Triangle( dc      : Drawing_Context;
                        x1, y1  : Integer;
                        x2, y2  : Integer;
                        x3, y3  : Integer;
                        color   : Color_Type;
                        opacity : Natural := 255 ) is
    begin
        if opacity < 255 then
            Set_Trans_Blender( 255, 255, 255, opacity );
            Drawing_Mode( DRAW_MODE_TRANS );
            Triangle( dc.bmp,
                      x1 - dc.offx, y1 - dc.offy,
                      x2 - dc.offx, y2 - dc.offy,
                      x3 - dc.offx, y3 - dc.offy,
                      color );
            Solid_Mode;
        else
            Triangle( dc.bmp,
                      x1 - dc.offx, y1 - dc.offy,
                      x2 - dc.offx, y2 - dc.offy,
                      x3 - dc.offx, y3 - dc.offy,
                      color );
        end if;
    end Triangle;

end Drawing_Contexts;
