
with Events;

pragma Elaborate_All( Events );

package Events.Application is

    APP_BLUR_ID      : constant Event_Id := To_Event_Id( "App_Blur" );
    APP_FOCUS_ID     : constant Event_Id := To_Event_Id( "App_Focus" );
    CLOSE_REQUEST_ID : constant Event_Id := To_Event_Id( "Close_Request" );
    CLOSE_WINDOW_ID  : constant Event_Id := To_Event_Id( "Close_Window" );

    -- A notification that the application window has lost focus.
    type App_Blur_Event is new Event with private;

    -- A notification that the application window has regained focus.
    type App_Focus_Event is new Event with private;

    -- A command to attempt to close the application. This does not have to be
    -- obeyed, for instance if the player hasn't saved his game yet.
    type Close_Request_Event is new Event with private;

    -- A command to close the application, no questions asked. This event is
    -- queued when it's time for the application to quit.
    type Close_Window_Event is new Event with private;

    ----------------------------------------------------------------------------

    -- Queues an App_Blur_Event.
    procedure Queue_App_Blur;

    -- Queues an App_Focus_Event.
    procedure Queue_App_Focus;

    -- Queues a Close_Request_Event.
    procedure Queue_Close_Request;

    -- Queues a Close_Window_Event.
    procedure Queue_Close_Window;

private

    type App_Blur_Event is new Event with null record;

    procedure Construct( this : access App_Blur_Event );

    ----------------------------------------------------------------------------

    type App_Focus_Event is new Event with null record;

    procedure Construct( this : access App_Focus_Event );

    ----------------------------------------------------------------------------

    type Close_Request_Event is new Event with null record;

    procedure Construct( this : access Close_Request_Event );

    ----------------------------------------------------------------------------

    type Close_Window_Event is new Event with null record;

    procedure Construct( this : access Close_Window_Event );

end Events.Application;
