
with Events.Manager;

package body Events.Game is

    not overriding
    procedure Construct( this : access Game_Paused_Event; paused : Boolean ) is
    begin
        Event(this.all).Construct( "Game_Paused" );
        this.paused := paused;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Paused( this : not null access Game_Paused_Event'Class ) return Boolean is
    begin
        return this.paused;
    end Is_Paused;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Game_Paused_Event ) return String is
    begin
        return "<" & this.Get_Name & " - paused: " & Boolean'Image( this.paused ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Game_State_Event; playing : Boolean ) is
    begin
        Event(this.all).Construct( "Game_State" );
        this.playing := playing;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Playing( this : not null access Game_State_Event'Class ) return Boolean is
    begin
        return this.playing;
    end Is_Playing;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Game_State_Event ) return String is
    begin
        return "<" & this.Get_Name & " - playing: " & Boolean'Image( this.playing ) & ">";
    end To_String;

    --==========================================================================

    overriding
    procedure Adjust( this : access Game_Var_Changed_Event ) is
    begin
        Event(this.all).Adjust;
        this.var := To_Unbounded_String( To_String( this.var ) );
        this.val := Copy( this.val );
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Game_Var_Changed_Event;
                         var  : String;
                         val  : in out A_Value ) is
    begin
        Event(this.all).Construct( "Game_Var_Changed" );
        this.var := To_Unbounded_String( var );
        this.val := val;
        val := null;
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    procedure Copy_Value( this : access Game_Var_Changed_Event;
                          val  : in out A_Value ) is
    begin
        val := Copy( this.val );
    end Copy_Value;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Game_Var_Changed_Event ) is
    begin
        Delete( this.val );
        Event(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Get_Var( this : not null access Game_Var_Changed_Event'Class ) return String is
    begin
        return To_String( this.var );
    end Get_Var;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Game_Var_Changed_Event ) return String is
    begin
        return "<" & this.Get_Name & " - var: " & To_String( this.var ) &
               ", val: " & To_String( this.val ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Loading_Event; loading : Boolean ) is
    begin
        Event(this.all).Construct( "Loading" );
        this.loading := loading;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Loading( this : not null access Loading_Event'Class ) return Boolean is
    begin
        return this.loading;
    end Is_Loading;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Loading_Event ) return String is
    begin
        return "<" & this.Get_Name & " - loading: " & Boolean'Image( this.loading ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Pause_Game_Event; paused : Boolean ) is
    begin
        Event(this.all).Construct( "Pause_Game" );
        this.paused := paused;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Paused( this : not null access Pause_Game_Event'Class ) return Boolean is
    begin
        return this.paused;
    end Is_Paused;

    --==========================================================================

    not overriding
    procedure Construct( this : access Scroll_View_Event; x, y : Float ) is
    begin
        Event(this.all).Construct( "Scroll_View" );
        this.x := x;
        this.y := y;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Scroll_View_Event'Class ) return Float is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Scroll_View_Event'Class ) return Float is
    begin
        return this.y;
    end Get_Y;

    --==========================================================================

    not overriding
    procedure Construct( this : access View_Ready_Event; ready : Boolean ) is
    begin
        Event(this.all).Construct( "View_Ready" );
        this.ready := ready;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Ready( this : not null access View_Ready_Event'Class ) return Boolean is
    begin
        return this.ready;
    end Is_Ready;

    --==========================================================================

    procedure Queue_End_Game is
        evt : A_End_Game_Event := new End_Game_Event;
    begin
        evt.Construct( "End_Game" );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_End_Game;

    ----------------------------------------------------------------------------

    procedure Queue_Game_Paused( paused : Boolean ) is
        evt : A_Game_Paused_Event := new Game_Paused_Event;
    begin
        evt.Construct( paused );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Game_Paused;

    ----------------------------------------------------------------------------

    procedure Queue_Game_State( playing : Boolean ) is
        evt : A_Game_State_Event := new Game_State_Event;
    begin
        evt.Construct( playing );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Game_State;

    ----------------------------------------------------------------------------

    procedure Queue_Game_Var_Changed( var : String;
                                      val : in out A_Value ) is
        evt : A_Game_Var_Changed_Event := new Game_Var_Changed_Event;
    begin
        evt.Construct( var, val );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Game_Var_Changed;

    ----------------------------------------------------------------------------

    procedure Queue_Loading( loading : Boolean ) is
        evt : A_Loading_Event := new Loading_Event;
    begin
        evt.Construct( loading );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Loading;

    ----------------------------------------------------------------------------

    procedure Queue_New_Game is
        evt : A_New_Game_Event := new New_Game_Event;
    begin
        evt.Construct( "New_Game" );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_New_Game;

    ----------------------------------------------------------------------------

    procedure Queue_Pause_Game( paused : Boolean ) is
        evt : A_Pause_Game_Event := new Pause_Game_Event;
    begin
        evt.Construct( paused );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Pause_Game;

    ----------------------------------------------------------------------------

    procedure Queue_Player_Died is
        evt : A_Player_Died_Event := new Player_Died_Event;
    begin
        evt.Construct( "Player_Died" );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Player_Died;

    ----------------------------------------------------------------------------

    procedure Queue_Scroll_View( x, y : Float ) is
        evt : A_Scroll_View_Event := new Scroll_View_Event;
    begin
        evt.Construct( x, y );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Scroll_View;

    ----------------------------------------------------------------------------

    procedure Queue_View_Ready( ready : Boolean ) is
        evt : A_View_Ready_Event := new View_Ready_Event;
    begin
        evt.Construct( ready );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_View_Ready;

end Events.Game;
