
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Events.Manager;
with Objects;

pragma Warnings( Off, Objects );
pragma Elaborate_All( Objects );

package body Events.Input is

    not overriding
    procedure Construct( this      : access Key_Event;
                         evtName   : String;
                         key       : Positive;
                         char      : Character;
                         modifiers : Modifiers_Array ) is
    begin
        Event(this.all).Construct( evtName );
        this.key := key;
        this.char := char;
        this.modifiers := modifiers;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Char( this : not null access Key_Event'Class ) return Character is
    begin
        return this.char;
    end Get_Char;

    ----------------------------------------------------------------------------

    function Get_Key( this : not null access Key_Event'Class ) return Positive is
    begin
        return this.key;
    end Get_Key;

    ----------------------------------------------------------------------------

    procedure Get_Modifiers( this      : not null access Key_Event'Class;
                             modifiers : out Modifiers_Array ) is
    begin
        modifiers := this.modifiers;
    end Get_Modifiers;

    ----------------------------------------------------------------------------

    function Is_Readable( this : not null access Key_Event'Class ) return Boolean is
    begin
        return this.char /= ASCII.NUL;
    end Is_Readable;

    ----------------------------------------------------------------------------

    function No_Modifiers( this : not null access Key_Event'Class ) return Boolean is
    begin
        return None( this.modifiers );
    end No_Modifiers;

    ----------------------------------------------------------------------------

    function Only_Alt( this : not null access Key_Event'Class ) return Boolean is
    begin
        return Only_Alt( this.modifiers );
    end Only_Alt;

    ----------------------------------------------------------------------------

    function Only_Ctrl( this : not null access Key_Event'Class ) return Boolean is
    begin
        return Only_Ctrl( this.modifiers );
    end Only_Ctrl;

    ----------------------------------------------------------------------------

    function Only_Shift( this : not null access Key_Event'Class ) return Boolean is
    begin
        return Only_Shift( this.modifiers );
    end Only_Shift;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Key_Event ) return String is

        function Get_Ctrl return String is
        begin
            if this.modifiers(CTRL) then
                return "Ctrl + ";
            end if;
            return "";
        end Get_Ctrl;

        function Get_Alt return String is
        begin
            if this.modifiers(ALT) then
                return "Alt + ";
            end if;
            return "";
        end Get_Alt;

        function Get_Shift return String is
        begin
            if this.modifiers(SHIFT) then
                return "Shift + ";
            end if;
            return "";
        end Get_Shift;

    begin
        return "<" & this.Get_Name & " - " & Get_Ctrl & Get_Alt & Get_Shift &
               Scancode_To_Name( this.key ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this    : access Mouse_Event;
                         evtName : String;
                         x, y    : Integer ) is
    begin
        Event(this.all).Construct( evtName );
        this.x := x;
        this.y := y;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Mouse_Event'Class ) return Integer is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Mouse_Event'Class ) return Integer is
    begin
        return this.y;
    end Get_Y;

    ----------------------------------------------------------------------------

    procedure Set_XY( this : not null access Mouse_Event'Class; x, y : Integer ) is
    begin
        this.x := x;
        this.y := y;
    end Set_XY;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Mouse_Event ) return String is
    begin
        return "<" & this.Get_Name & " - x:" & Integer'Image( this.x ) &
               ", y:" & Integer'Image( this.y ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this      : access Mouse_Button_Event;
                         evtName   : String;
                         x, y      : Integer;
                         btn       : Mouse_Button;
                         modifiers : Modifiers_Array ) is
    begin
        Mouse_Event(this.all).Construct( evtName, x, y );
        this.btn := btn;
        this.modifiers := modifiers;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Button( this : not null access Mouse_Button_Event'Class ) return Mouse_Button is
    begin
        return this.btn;
    end Get_Button;

    ----------------------------------------------------------------------------

    function Get_Modifiers( this : not null access Mouse_Button_Event'Class ) return Modifiers_Array is
    begin
        return this.modifiers;
    end Get_Modifiers;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Mouse_Button_Event ) return String is
    begin
        return "<" & this.Get_Name & " - x:" & Integer'Image( this.x ) &
               ", y:" & Integer'Image( this.y ) &
               ", btn: " & To_String( this.btn ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this    : access Mouse_Scroll_Event;
                         evtName : String;
                         x, y    : Integer;
                         amount  : Integer ) is
    begin
        Mouse_Event(this.all).Construct( evtName, x, y );
        this.amount := amount;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Amount( this : not null access Mouse_Scroll_Event'Class ) return Integer is
    begin
        return this.amount;
    end Get_Amount;

    --==========================================================================

    procedure Queue_Key_Held( key       : Positive;
                              modifiers : Modifiers_Array;
                              capslock  : Boolean ) is
        evt : A_Key_Event := new Key_Held_Event;
    begin
        evt.Construct( "Key_Held", key,
                       Scancode_To_Char( key, modifiers(SHIFT) xor capslock ),
                       modifiers );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Key_Held;

    ----------------------------------------------------------------------------

    procedure Queue_Key_Press( key       : Positive;
                               modifiers : Modifiers_Array;
                               capslock  : Boolean ) is
        evt : A_Key_Event := new Key_Press_Event;
    begin
        evt.Construct( "Key_Press", key,
                       Scancode_To_Char( key, modifiers(SHIFT) xor capslock ),
                       modifiers );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Key_Press;

    ----------------------------------------------------------------------------

    procedure Queue_Key_Release( key       : Positive;
                                 modifiers : Modifiers_Array;
                                 capslock  : Boolean ) is
        evt : A_Key_Event := new Key_Release_Event;
    begin
        evt.Construct( "Key_Release", key,
                       Scancode_To_Char( key, modifiers(SHIFT) xor capslock ),
                       modifiers );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Key_Release;

    ----------------------------------------------------------------------------

    procedure Queue_Mouse_Click( x, y : Integer; btn : Mouse_Button ) is
        evt : A_Mouse_Button_Event := new Mouse_Click_Event;
    begin
        evt.Construct( "Mouse_Click", x, y, btn, MODIFIERS_NONE );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Mouse_Click;

    ----------------------------------------------------------------------------

    procedure Queue_Mouse_Doubleclick( x, y : Integer; btn : Mouse_Button ) is
        evt : A_Mouse_Button_Event := new Mouse_Doubleclick_Event;
    begin
        evt.Construct( "Mouse_Doubleclick", x, y, btn, MODIFIERS_NONE );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Mouse_Doubleclick;

    ----------------------------------------------------------------------------

    procedure Queue_Mouse_Held( x, y : Integer; btn : Mouse_Button ) is
        evt : A_Mouse_Button_Event := new Mouse_Held_Event;
    begin
        evt.Construct( "Mouse_Held", x, y, btn, MODIFIERS_NONE );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Mouse_Held;

    ----------------------------------------------------------------------------

    procedure Queue_Mouse_Move( x, y : Integer ) is
        evt : A_Mouse_Event := new Mouse_Held_Event;
    begin
        evt.Construct( "Mouse_Move", x, y );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Mouse_Move;

    ----------------------------------------------------------------------------

    procedure Queue_Mouse_Press( x, y      : Integer;
                                 btn       : Mouse_Button;
                                 modifiers : Modifiers_Array ) is
        evt : A_Mouse_Button_Event := new Mouse_Press_Event;
    begin
        evt.Construct( "Mouse_Press", x, y, btn, modifiers );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Mouse_Press;

    ----------------------------------------------------------------------------

    procedure Queue_Mouse_Release( x, y : Integer; btn : Mouse_Button ) is
        evt : A_Mouse_Button_Event := new Mouse_Release_Event;
    begin
        evt.Construct( "Mouse_Release", x, y, btn, MODIFIERS_NONE );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Mouse_Release;

    ----------------------------------------------------------------------------

    function Create_Mouse_Release( x, y : Integer;
                                   btn  : Mouse_Button ) return A_Mouse_Button_Event is
        evt : constant A_Mouse_Button_Event := new Mouse_Release_Event;
    begin
        evt.Construct( "Mouse_Release", x, y, btn, MODIFIERS_NONE );
        return evt;
    end Create_Mouse_Release;

    ----------------------------------------------------------------------------

    procedure Queue_Mouse_Scroll( x, y : Integer; amount : Integer ) is
        evt : A_Mouse_Scroll_Event := new Mouse_Scroll_Event;
    begin
        evt.Construct( "Mouse_Scroll", x, y, amount );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Mouse_Scroll;

end Events.Input;
