
with Allegro.Text_IO;
with GNAT.OS_Lib;                       use GNAT.OS_Lib;
with Preferences;                       use Preferences;
with Support.Paths;                     use Support.Paths;
with System;                            use System;

package body Font_API.Allegro_Fonts is

    function Load_Font( filename : String; size : Positive ) return Font_Type is
        this : Font_Type := new Allegro_Font;
    begin
        Allegro_Font(this.all).Construct( filename, size );
        return this;
    exception
        when others =>
            Delete( this );
            return null;
    end Load_Font;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this     : access Allegro_Font;
                         filename : String;
                         size     : Positive ) is
    begin
        Abstract_Font(this.all).Construct;
        this.height := size;
        if Is_Regular_File( filename ) then
            this.ptr := Allegro.Fonts.Load_Font( filename, null, Null_Address );
        elsif Is_Regular_File( Get_Pref( "application", "media" ) & Get_Filename( filename ) ) then
            this.ptr := Allegro.Fonts.Load_Font( Get_Pref( "application", "media" ) & Get_Filename( filename ),
                                                 null, Null_Address );
        end if;
        if this.ptr = null then
            raise FONT_EXCEPTION with "Error loading font <" & filename & '>';
        end if;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Allegro_Font ) is
    begin
        if this.ptr /= null then
            Allegro.Fonts.Destroy_Font( this.ptr );
        end if;
        Abstract_Font(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Text_Height( this : access Allegro_Font ) return Positive is
    begin
        return Positive'Max( this.height, Allegro.Text_IO.Text_Height( this.ptr ) );
    end Text_Height;

    ----------------------------------------------------------------------------

    overriding
    function Text_Length( this : access Allegro_Font;
                          str  : String ) return Natural is
    begin
        return Allegro.Text_IO.Text_Length( this.ptr, str );
    end Text_Length;

    ----------------------------------------------------------------------------

    overriding
    procedure Textout( this   : access Allegro_Font;
                       bmp    : A_Bitmap;
                       str    : String;
                       x, y   : Integer;
                       color  : Color_Type;
                       smooth : Boolean ) is
        pragma Unreferenced( smooth );
    begin
        Allegro.Text_IO.Textout_Ex( bmp, this.ptr, str, x, y, color, -1 );
    end Textout;

end Font_API.Allegro_Fonts;
