
with Widgets.Containers.Windows;        use Widgets.Containers.Windows;

package body Widgets.Containers.Panels.Popups is

    function Create_Popup( view  : not null access Game_Views.Game_View'Class;
                           id    : String;
                           title : String := "";
                           icon  : String := "" ) return A_Popup is
        this : A_Popup := new Popup;
    begin
        this.Construct( view, id, title, icon );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Popup;

    ----------------------------------------------------------------------------

    overriding
    procedure Add( this    : access Popup;
                   child   : in out A_Widget;
                   consume : Boolean := True ) is
    begin
        if child.all in Menu_Item'Class then
            A_Menu_Item(child).Add_Listener( A_Menu_Listener(this) );
            Panel(this.all).Add( child, consume );
        else
            raise Constraint_Error with "Only a Menu_Item may be added to a Popup menu";
        end if;
    end Add;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this  : access Popup;
                         view  : not null access Game_Views.Game_View'Class;
                         id    : String;
                         title : String;
                         icon  : String ) is
    begin
        Panel(this.all).Construct( view, id, title, icon );
        this.visible := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Height( this : access Popup ) return Natural is
        height : Integer := 0;

        procedure Examine( pos : Widget_Lists.Cursor ) is
        begin
            height := height + Widget_Lists.Element( pos ).Get_Min_Height;
        end Examine;

    begin
        if this.minHeight > 0 then
            return this.minHeight;
        else
            this.children.Iterate( Examine'Access );
            return this.contentTop + height + Themes.Get.Border_Width;
        end if;
    end Get_Min_Height;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Width( this : access Popup ) return Natural is
        width : Integer := 0;

        procedure Examine( pos : Widget_Lists.Cursor ) is
            w : constant Integer := Widget_Lists.Element( pos ).Get_Min_Width;
        begin
            if w > width then
                width := w;
            end if;
        end Examine;

    begin
        if this.minWidth > 0 then
            return this.minWidth;
        else
            this.children.Iterate( Examine'Access );
            return width + Themes.Get.Border_Width * 2;
        end if;
    end Get_Min_Width;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Popup;
                             action : A_Menu_Action ) is
    begin
        if action.Get_Id = Menu_Selected or else action.Get_Id = Menu_Unselected then
            -- pops this widget off the window's popup stack which will cause this
            -- widget to be hidden.
            this.Get_Window.Pop_Popup;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Pack( this : access Popup ) is
        top : Integer := 0;

        procedure Examine( pos : Widget_Lists.Cursor ) is
            child   : constant A_Widget := Widget_Lists.Element( pos );
            cHeight : constant Integer := child.Get_Min_Height;
        begin
            child.Set_Layout_LTRB( 0, top, 0, -(top + cHeight - 1) );
            top := top + cHeight;
        end Examine;

    begin
        this.children.Iterate( Examine'Access );
        Panel(this.all).Pack;
    end Pack;

end Widgets.Containers.Panels.Popups;
