
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Themes.Palette;                    use Themes.Palette;

package body Widgets.Labels is

    function Create_Label( view : not null access Game_Views.Game_View'Class;
                           id   : String;
                           text : String := "";
                           icon : String := "" ) return A_Label is
        this : A_Label := new Label;
    begin
        this.Construct( view, id, text, icon );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Label;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Label;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String;
                         text : String;
                         icon : String ) is
    begin
        Widget(this.all).Construct( view, id );
        this.acceptFocus := False;
        this.Set_Text( text );
        this.Set_Icon( icon );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Label; dc : Drawing_Context ) is
        theme : constant A_Theme := Themes.Get;
        icon  : constant A_Bitmap := theme.Get_Bitmap( this.icon );
        bw    : constant Integer := theme.Border_Width;
        pad   : Integer := 0;
    begin
        if this.colors(Foreground) /= base_colors(Transparent) then
            Rectfill( dc, 0, 0, this.cwidth - 1, this.cheight - 1, this.colors(Foreground) );
        end if;
        if this.border /= Border_None then
            pad := bw + SPACING;
        end if;
        theme.Draw_Label( dc, pad, pad,
                          this.cwidth - 1 - pad, this.cheight - 1 - pad,
                          icon, To_String( this.text ), To_String( this.fontName ),
                          this.fontSize, this.colors(Text), this.align,
                          smooth => this.fontSmooth and then not this.transparent );
        if this.border /= Border_None then
            theme.Draw_Border( dc, this.border,
                               0, 0, this.cwidth - 1, this.cheight - 1,
                               this.colors(Foreground) );
        end if;
    end Draw_Content;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Height( this : access Label ) return Natural is
        theme : A_Theme;
        pad   : Natural := 0;
    begin
        if this.minHeight > 0 then
            return this.minHeight;
        else
            theme := Themes.Get;
            if this.border /= Border_None then
                pad := theme.Border_Width * 2 + SPACING * 2;
            end if;
            return pad + theme.Label_Height( theme.Get_Bitmap( this.icon ),
                                             To_String( this.text ),
                                             To_String( this.fontName ),
                                             this.fontSize );
        end if;
    end Get_Min_Height;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Width( this : access Label ) return Natural is
        theme : A_Theme;
        pad   : Natural := 0;
    begin
        if this.minWidth > 0 then
            return this.minWidth;
        else
            theme := Themes.Get;
            if this.border /= Border_None then
                pad := theme.Border_Width * 2 + SPACING * 2;
            end if;
            return pad + theme.Label_Width( theme.Get_Bitmap( this.icon ),
                                            To_String( this.text ),
                                            To_String( this.fontName ),
                                            this.fontSize );
        end if;
    end Get_Min_Width;

    ----------------------------------------------------------------------------

    function Get_Text( this : not null access Label'Class ) return String is
    begin
        return To_String( this.text );
    end Get_Text;

    ----------------------------------------------------------------------------

    procedure Set_Align( this : not null access Label'Class; align : Align_Type ) is
    begin
        if align /= this.align then
            this.align := align;
            A_Widget(this).Set_Dirty;
        end if;
    end Set_Align;

    ----------------------------------------------------------------------------

    overriding
    procedure Set_Color( this    : access Label;
                         purpose : Color_Purpose;
                         color   : Color_Type ) is
    begin
        Widget(this.all).Set_Color( purpose, color );
        this.transparent := purpose = Foreground and then
                            color = base_colors(Transparent);
    end Set_Color;

    ----------------------------------------------------------------------------

    procedure Set_Icon( this : not null access Label'Class; icon : String ) is
    begin
        if icon'Length > 0 then
            this.icon := Themes.Get.Get_ID( icon );
            pragma Debug( this.icon = 0,
                          Dbg( "Warning: Icon <" & icon & "> not found for widget " &
                               To_String( this ),
                               D_GUI, WARNING ) );
        else
            this.icon := 0;
        end if;
        A_Widget(this).Set_Dirty;
    end Set_Icon;

    ----------------------------------------------------------------------------

    procedure Set_Text( this : not null access Label'Class; text : String ) is
    begin
        this.text := To_Unbounded_String( text );
        A_Widget(this).Set_Dirty;
    end Set_Text;

end Widgets.Labels;
