
with Allegro.Bitmaps;                   use Allegro.Bitmaps;
with Allegro.Truecolor;                 use Allegro.Truecolor;
with Applications;
--with Debugging;                         use Debugging;
with Preferences;                       use Preferences;
with Resources.Images;                  use Resources.Images;
with Support.Paths;                     use Support.Paths;

pragma Elaborate_All( Applications );
pragma Elaborate_All( Preferences );
pragma Elaborate_All( Support.Paths );

package body Applications.Gui.Games.Ked is

    function Create_Application return A_Application is
        this : constant A_Application := new Ked_Application;
    begin
        this.Construct;
        return this;
    end Create_Application;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Ked_Application ) is
    begin
        Game_Application(this.all).Construct( company  => "Kevin Wellwood",
                                              name     => "ked",
                                              winTitle => "Keen Editor" );
        this.interactive := True;
        this.use_mouse := True;
        Set_Working_Directory;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Loading_Screen( this : access Ked_Application;
                                   dc   : Drawing_Context ) is
        pragma Unreferenced( this );
        loading : A_Bitmap;
    begin
        loading := Load_Image( "loading.png", "resources" );
        if loading /= null then
            Clear_To_Color( dc, Makecol( 92, 92, 92 ) );
            Draw_Alpha_Sprite( dc, loading,
                               (Get_Width( dc ) - Get_Width( loading )) / 2,
                               (Get_Height( dc ) - Get_Height( loading )) / 2 );
            Destroy_Bitmap( loading );
        end if;
    end Draw_Loading_Screen;

    ----------------------------------------------------------------------------

    overriding
    function Init( this : access Ked_Application ) return Boolean is
    begin
        return this.Init( 800, 500, 1, app_windowed => True );
    end Init;

begin

    Register_Allocator( Create_Application'Access );

    Preferences.Set_Default( "application", "max_input_fps", 80 );
    Preferences.Set_Default( "application", "max_logic_fps", 80 );
    Preferences.Set_Default( "application", "max_render_fps", 30 );
    Preferences.Set_Default( "application", "editing_mode", True );
    Preferences.Set_Default( "application", "license", "credits" & Slash );
    Preferences.Set_Default( "application", "media", App_Data_Directory &
                                                     "Wellwood" & Slash &
                                                     "Commander Keen 7" & Slash &
                                                     "media" & Slash );

end Applications.Gui.Games.Ked;
