
with Actions;                           use Actions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Keyboard;                          use Keyboard;
with Preferences;                       use Preferences;
with Support;                           use Support;
with Themes;                            use Themes;
with Themes.Palette;                    use Themes.Palette;
with Widgets.Buttons;                   use Widgets.Buttons;
with Widgets.Buttons.Checkboxes;        use Widgets.Buttons.Checkboxes;
with Widgets.Buttons.Toggles;           use Widgets.Buttons.Toggles;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Containers.Scenes.Ked;     use Widgets.Containers.Scenes.Ked;

package body Game_Views.Ked.Toolbar.Layers_Menu is

    btnSize   : constant := 22;
    rowHeight : constant := 20;

    ----------------------------------------------------------------------------

    procedure Shortcut_Set_Active_Layer( action  : A_Key_Action;
                                         handled : out Boolean ) is
        layer : Integer := 0;
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            if action.Get_Key >= KEY_1 and then action.Get_Key <= KEY_9 then
                layer := action.Get_Key - KEY_1 + 1;
                A_Button(action.Get_Source.Get_View.Get_Widget( "btnLayer" & Image( layer ) )).Set_State( True );
                handled := True;
            end if;
        end if;
    exception
        when others =>
            -- a button for the layer doesn't exist
            null;
    end Shortcut_Set_Active_Layer;

    ----------------------------------------------------------------------------

    procedure Shortcut_Show_Layer( action  : A_Key_Action;
                                   handled : out Boolean ) is
        layer : Integer := 0;
        chk   : A_Button;
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            if action.Get_Key >= KEY_1 and then action.Get_Key <= KEY_9 then
                layer := action.Get_Key - KEY_1 + 1;
                chk := A_Button(action.Get_Source.Get_View.Get_Widget( "chkLayer" & Image( layer ) ));
                if chk.Is_Enabled then
                    chk.Toggle_State;
                end if;
                handled := True;
            end if;
        end if;
    exception
        when ID_NOT_FOUND =>
            -- a checkbox for the layer doesn't exist
            null;
    end Shortcut_Show_Layer;

    ----------------------------------------------------------------------------

    procedure Make_Menu( view : not null A_Ked_View;
                         win  : not null A_Window ) is
        button  : A_Button;
        panel   : A_Panel := Create_Panel( view, "pnlLayers" );
    begin
        button := Create_Toggle_Button( view, "btnLayer", icon => "layer_icon" );
        button.Set_Layout_LTRB( -(btnSize-1), -(btnSize-1), 0, 0 );
        button.Set_Focusable( False );
        button.Set_Enabled( False );
        button.Add_Listener( Show_Layers'Access );
        win.Add( A_Widget(button) );

        view.layerGrp := Create_Button_Group;
        panel.Set_Layout_LTRB( 0, 0, -1, -1 );
        panel.Set_Visible( False );
        win.Add( A_Widget(panel) );

        win.Add_Listener( KEY_1, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_2, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_3, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_4, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_5, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_6, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_7, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_8, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_9, (others=>No), Shortcut_Set_Active_Layer'Access );
        win.Add_Listener( KEY_1, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_2, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_3, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_4, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_5, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_6, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_7, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_8, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
        win.Add_Listener( KEY_9, (CTRL=>Yes, others=>No), Shortcut_Show_Layer'Access );
    end Make_Menu;

    ----------------------------------------------------------------------------

    procedure Update_Menu( view   : not null access Ked_View'Class;
                           layers : not null A_Layer_Array ) is
        theme      : constant A_Theme := Themes.Get;
        layerPanel : constant A_Panel := A_Panel(view.Get_Widget( "pnlLayers" ));
    begin
        view.layerGrp.Clear;
        layerPanel.Delete_Children;
        layerPanel.Set_Layout_LTRB( -150, -(btnSize + (layers'Length + 1) * rowHeight + 1), 0, btnSize );
        view.Get_Widget( "btnLayer" ).Set_Enabled( True );

        for i in layers'Range loop
            declare
                btn : A_Button := Create_Toggle_Button( view, "btnLayer" & Image( i ),
                                                        "Layer " & Image( i ) );
                chk : A_Checkbox := Create_Checkbox( view, "chkLayer" & Image( i ) );
                top : constant Integer := (layers'Last - i + 1) * rowHeight;
            begin
                -- create the active button
                btn.Set_Focusable( False );
                btn.Set_Attribute( "layer", i );
                btn.Set_Border( Border_None );
                btn.Contrast_Text( True );
                btn.Set_Layout_LTRB( 0, top, 0, -(top + rowHeight - 1) );
                if i = layers'First then
                    btn.Set_State( True );
                    btn.Set_Color( Foreground, theme.Color(Selected) );
                end if;
                btn.Add_Listener( Set_Active_Layer'Access );
                view.layerGrp.Add( btn );
                layerPanel.Add( A_Widget(btn) );

                -- create the visibility checkbox
                chk.Set_Focusable( False );
                chk.Set_Attribute( "layer", i );
                chk.Set_Align( Align_Center );
                chk.Set_Check_Icon( "eye_icon" );
                chk.Set_Layout_LTRB( 2, top, -15, -(top + rowHeight - 1) );
                chk.Set_State( True );
                chk.Set_Color( Foreground, base_colors(Transparent) );
                chk.Set_Border( Border_Line );
                chk.Add_Listener( Show_Layer'Access );
                layerPanel.Add( A_Widget(chk) );
            end;
        end loop;

        declare
            btn : A_Button := Create_Toggle_Button( view, "btnLayerClipping", "Show Clipping" );
        begin
            -- create the visibility button for clipping layer
            btn.Set_Focusable( False );
            btn.Set_Border( Border_None );
            btn.Contrast_Text( True );
            btn.Set_Layout_LTRB( 0, 0, 0, -(rowHeight-1) );
            if Get_Pref( "draw_clipping" ) then
                btn.Set_State( True );
                btn.Set_Color( Foreground, theme.Color(Selected) );
            end if;
            btn.Add_Listener( Show_Clipping'Access );
            layerPanel.Add( A_Widget(btn) );
        end;
    end Update_Menu;

    ----------------------------------------------------------------------------

    procedure Show_Layers( action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            action.Get_Source.Get_View.Get_Widget( "pnlLayers" ).Set_Visible( True );
        elsif action.Get_Id = Release then
            action.Get_Source.Get_View.Get_Widget( "pnlLayers" ).Set_Visible( False );
        end if;
    end Show_Layers;

    ----------------------------------------------------------------------------

    -- Sets the currently active layer for tool operations
    procedure Set_Active_Layer( action : A_Button_Action ) is
        oldLayer : constant Integer := A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Get_Active_Layer;
        newLayer : constant Integer := action.Get_Source.Get_Attribute( "layer" );
    begin
        if action.Get_Id = Press then
            A_Button(action.Get_Source.Get_View.Get_Widget( "chkLayer" & Image( oldLayer ) )).Set_Enabled( True );
            A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Set_Active_Layer( newLayer );
            action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Selected ) );
            A_Button(action.Get_Source.Get_View.Get_Widget( "chkLayer" & Image( newLayer ) )).Set_State( True );
            A_Button(action.Get_Source.Get_View.Get_Widget( "chkLayer" & Image( newLayer ) )).Set_Enabled( False );
        elsif action.Get_Id = Release then
            action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Foreground ) );
        end if;
    end Set_Active_Layer;

    ----------------------------------------------------------------------------

    procedure Show_Clipping( action : A_Button_Action ) is
        state : Boolean;
    begin
        if action.Get_Id = Press or else action.Get_Id = Release then
            state := A_Button(action.Get_Source).Get_State;
            Set_Pref( "draw_clipping", state );
            action.Get_Source.Get_View.Get_Widget( "scene" ).Redraw;
            if state then
                action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Selected ) );
            else
                action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Foreground ) );
            end if;
        end if;
    end Show_Clipping;

    ----------------------------------------------------------------------------

    procedure Show_Layer( action : A_Button_Action ) is
        layer : constant Integer := action.Get_Source.Get_Attribute( "layer" );
    begin
        if action.Get_Id = Press then
            A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Set_Layer_Visible( layer, True );
        elsif action.Get_Id = Release then
            A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Set_Layer_Visible( layer, False );
        end if;
    end Show_Layer;

end Game_Views.Ked.Toolbar.Layers_Menu;
