
with Resources;                         use Resources;
with Themes;                            use Themes;

pragma Elaborate_All( Themes );

package body Themes.Ked is

    LIB_NAME : constant String := "icons";

    ----------------------------------------------------------------------------

    function Create_Theme return A_Theme is
        this : A_Theme := new Ked_Theme;
    begin
        this.Construct;
        return this;
    exception
        when others =>
            Delete( this );
            raise;
    end Create_Theme;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Ked_Theme ) is
    begin
        Theme(this.all).Construct;
        this.colors(Background)    := Makecol( 148, 145, 140 );
        this.colors(Foreground)    := Makecol( 212, 208, 200 );
        this.colors(Selected)      := Makecol(  58,  95, 205 );
        this.colors(Selected_Text) := Makecol( 255, 255, 255 );
        this.colors(Text)          := Makecol(   0,   0,   0 );
        this.filter := Filter_2xSaI;
        this.Register_Font( "dejavusans.ttf", DEFAULT_FONT_NAME );  -- may raise exception
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Ked_Theme ) is
    begin
        Unload_Library( this.lib );
        Theme(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Get_Bitmap( this : access Ked_Theme;
                         id   : Natural ) return A_Bitmap is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME );
        end if;
        if this.lib /= null then
            return this.lib.Get_Bitmap_Sync( id );
        end if;
        return null;
    end Get_Bitmap;

    ----------------------------------------------------------------------------

    overriding
    function Get_ID( this : access Ked_Theme; name : String ) return Natural is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME );
        end if;
        if this.lib /= null then
            return this.lib.Get_Id( name );
        else
            --return 0;
            raise RESOURCE_ERROR with "Error loading tile library <" & LIB_NAME & ">";
        end if;
    end Get_ID;

begin

    Register_Allocator( Create_Theme'Access );

end Themes.Ked;
