
with Allegro.Keyboard;                  use Allegro.Keyboard;
with GNAT.OS_Lib;                       use GNAT.OS_Lib;
with Game_Views;                        use Game_Views;
with Preferences;                       use Preferences;
with Support;                           use Support;
with Support.Paths;                     use Support.Paths;
with Version;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Containers.Windows;        use Widgets.Containers.Windows;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.About is

    function Create_About_Dialog( view : not null access Game_Views.Game_View'Class;
                                  id   : String ) return A_Dialog is
        this : A_Dialog := new About_Dialog;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_About_Dialog;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access About_Dialog;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
        button : A_Button;
        label  : A_Label;
    begin
        Dialog(this.all).Construct( view, id, "About Keen Editor", "" );
        this.Set_Layout_Center( width => 310, height => 140 );

        label := Create_Label( view, this.id & ":lblIcon", icon => "ked_icon" );
        label.Set_Layout_LTWH( 5, 5, 54, 54 );
        this.Add( A_Widget(label) );

        label := Create_Label( view, this.id & ":lblLine1", "Keen Editor" );
        label.Set_Layout_LTRB( 64, 5, 5, -20 );
        this.Add( A_Widget(label) );

        label := Create_Label( view, this.id & ":lblLine2", "v" & Version.Release );
        label.Set_Layout_LTRB( 64, 23, 5, -38 );
        this.Add( A_Widget(label) );

        label := Create_Label( view, this.id & ":lblLine3", "Kevin Wellwood (C) " & Version.Copyright_Year );
        label.Set_Layout_LTRB( 64, 41, 5, -56 );
        this.Add( A_Widget(label) );

        label := Create_Label( view, this.id & ":lblLine4", "Portions Copyright (C) by their respective authors." );
        label.Set_Layout_LTRB( 5, 69, 5, -84 );
        this.Add( A_Widget(label) );

        button := Create_Push_Button( view, this.id & ":btnLic", "Third-Party Licenses" );
        button.Set_Layout_LTWH( 5, -5, 130, 20 );
        button.Set_Prev( this.id & ":btnOk" );
        button.Set_Next( this.id & ":btnOk" );
        button.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnOk", "OK" );
        button.Set_Layout_LTWH( -5, -5, 55, 20 );
        button.Set_Prev( this.id & ":btnLic" );
        button.Set_Next( this.id & ":btnLic" );
        button.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(button) );
    exception
        when others =>
            Delete( A_Widget(button) );
            Delete( A_Widget(label) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access About_Dialog;
                             action : A_Button_Action ) is
    begin
        if action.Get_Id = Click then
            if action.Get_Source.Get_Id = String'(this.id & ":btnLic") then
                Reveal_Path( Normalize_Pathname( Get_Pref( "application", "license" ) ) );
            elsif action.Get_Source.Get_Id = String'(this.id & ":btnOk") then
                this.Hide;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access About_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        if action.Get_Key = KEY_ESC and then action.Get_Id = Key_Press then
            this.Hide;
            handled := True;
        else
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access About_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_Window.Give_Focus( this.Get_View.Get_Widget( this.id & ":btnOk" ) );
    end Show;

end Widgets.Containers.Panels.Dialogs.About;
