
with Actors.Keen;
with Ada.Numerics.Discrete_Random;
--with Debugging;                         use Debugging;
with Entities;                          use Entities;
with Events.Audio;                      use Events.Audio;
with Games;                             use Games;
with Worlds.Keen;                       use Worlds.Keen;
with Tiles.Keen;                        use Tiles.Keen;

pragma Elaborate_All( Games );

package body Games.Keen is

    MAP_WORLD      : constant String := "shadowlands";
    MAP_WORLD_NAME : constant String := "Shadowlands";

    ----------------------------------------------------------------------------

    function Next_Dialog_Id return Integer is
        package Random_32 is new Ada.Numerics.Discrete_Random( Integer );
        use Random_32;
        g : Generator;
    begin
        Reset( g );
        return Random( g );
    end Next_Dialog_Id;

    ----------------------------------------------------------------------------

    function Create_Game return A_Game is
        this : constant A_Game := new Keen_Game;
    begin
        this.Construct;
        return this;
    end Create_Game;

    ----------------------------------------------------------------------------

    overriding
    procedure Add_Event_Listeners( this : access Keen_Game ) is
    begin
        Game(this.all).Add_Event_Listeners;
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Dialog_Response" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Entity_Entered_Tile" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Give_Ammo" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Give_Drops" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Give_Points" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Impulse" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Player_Died" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Set_Tile" );
    end Add_Event_Listeners;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Keen_Game;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
    begin
        if evt.Get_Id = ENTITY_ENTERED_TILE_ID then
            this.Handle( A_Entity_Entered_Tile_Event(evt) );

        elsif evt.Get_Id = IMPULSE_ID then
            this.Handle( A_Impulse_Event(evt) );

        elsif evt.Get_Id = GIVE_POINTS_ID then
            this.Handle( A_Give_Points_Event(evt) );

        elsif evt.Get_Id = GIVE_DROPS_ID then
            this.Handle( A_Give_Drops_Event(evt) );

        elsif evt.Get_Id = GIVE_AMMO_ID then
            this.Handle( A_Give_Ammo_Event(evt) );

        elsif evt.Get_Id = SET_TILE_ID then
            this.Handle( A_Set_Tile_Event(evt) );

        elsif evt.Get_Id = PLAYER_DIED_ID then
            this.Handle( A_Player_Died_Event(evt) );

        elsif evt.Get_Id = DIALOG_RESPONSE_ID then
            this.Handle( A_Dialog_Response_Event(evt) );

        else
            Game(this.all).Handle_Event( evt, resp );
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Dialog_Response_Event ) is
    begin
        if evt.Get_Dialog_Id = this.dialogId then
            this.dialogId := Integer'First;
            if evt.Get_Choice = 1 then
                this.Load_World( this.world.Get_Filename );
            else
                this.Load_World( MAP_WORLD );
            end if;
            this.Pause( False );
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Entity_Entered_Tile_Event ) is
    begin
        if evt.Get_Id = this.world.Get_Player.Get_Id then
            --Dbg( "Player touched" & evt.Get_X'Img & "," & evt.Get_Y'Img );
            if A_Keen_World(this.world).Get_Action( evt.Get_X, evt.Get_Y ) = HAZZARD then
                this.world.Get_Player.Die;
            end if;
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Give_Ammo_Event ) is
    begin
        if evt.Get_Amount + this.Get_Game_Var( "ammo" ) >= 0 then
            this.Game_Var_Add( "ammo", evt.Get_Amount );
        else
            -- constrain to a minimum of zero
            this.Game_Var_Add( "ammo", 0 );
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Give_Drops_Event ) is
        drops : Integer :=  this.Get_Game_Var( "drops" );
    begin
        drops := drops + evt.Get_Amount;
        while drops >= 100 loop
            drops := drops - 100;
            this.One_Up;
        end loop;
        this.Set_Game_Var( "drops", drops );
    end Handle;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Give_Points_Event ) is
    begin
        this.Game_Var_Add( "points", evt.Get_Amount );
        if this.Get_Game_Var( "points" ) >= this.Get_Game_Var( "nextLife" ) then
            this.One_Up;

            -- The value of 'nextLife' doubles each time
            this.Game_Var_Add( "nextLife", this.Get_Game_Var( "nextLife" ) );
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Impulse_Event ) is
        crsr : Actor_Maps.Cursor;
    begin
        if not this.paused then
            crsr := this.actorIndex.Find( evt.Get_Id );
            if Actor_Maps.Has_Element( crsr ) then
                Actor_Maps.Element( crsr ).Impulse( evt.Get_Impulse_Name );
            end if;
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Player_Died_Event ) is
        pragma Unreferenced( evt );
        dialog : A_Dialog_Desc;
    begin
        if this.Get_Game_Var( "lives" ) > 0 then
            -- display the level retry dialog
            this.Game_Var_Add( "lives", -1 );
            this.Pause( True );

            dialog := Create_Dialog_Desc( this.world.Get_Title );
            dialog.Add_Choice( "Try Again" );
            dialog.Add_Choice( "Exit to " & MAP_WORLD_NAME );
            this.dialogId := Next_Dialog_Id;
            Queue_Dialog( this.dialogId, dialog );
        else
            -- all out of lives, game over!
            Queue_End_Game;
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Set_Tile_Event ) is
    begin
        if this.world /= null then
            this.world.Set_Tile( evt.Get_Layer,
                                 evt.Get_X, evt.Get_Y,
                                 evt.Get_Tile_ID,
                                 notify => True );
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    overriding
    procedure Load_World( this : access Keen_Game; name : String ) is
    begin
        if name'Length = 0 then
            Game(this.all).Load_World( MAP_WORLD );
        else
            Game(this.all).Load_World( name );
        end if;
    end Load_World;

    ----------------------------------------------------------------------------

    overriding
    procedure New_Game( this : access Keen_Game ) is
    begin
        Game(this.all).New_Game;

        -- setup initial game state
        this.Set_Game_Var( "ammo",     5 );
        this.Set_Game_Var( "drops",    0 );
        this.Set_Game_Var( "points",   0 );
        this.Set_Game_Var( "nextLife", 20_000 );
        this.Set_Game_Var( "lives",    3 );
        this.Set_Game_Var( "ancients", 0 );
        this.Set_Game_Var( "scuba",    False );

        -- levels that have not yet been beaten
        this.Set_Game_Var( "level.border", False );

        -- create player with initial game state
        declare
            player : constant A_Actor := Actors.Keen.Create_Keen;
        begin
            player.Set_Game( A_Game(this) );
            this.actors.Append( player );
        end;

        -- load the first world
        this.Load_World( MAP_WORLD );
    end New_Game;

    ----------------------------------------------------------------------------

    procedure One_Up( this : not null access Keen_Game'Class ) is
    begin
        this.Game_Var_Add( "lives", 1 );
        Queue_Play_Sound( "get_1up" );
    end One_Up;

    ----------------------------------------------------------------------------

    overriding
    procedure Pause( this : access Keen_Game; enabled : Boolean ) is
        crsr : Actor_Maps.Cursor;
    begin
        Game(this.all).Pause( enabled );
        if enabled then
            -- pause the actor, stopping any on-going actions keen's doing
            crsr := this.actorIndex.Find( this.world.Get_Player.Get_Id );
            if Actor_Maps.Has_Element( crsr ) then
                Actor_Maps.Element( crsr ).Pause;
            end if;
        end if;
    end Pause;

    ----------------------------------------------------------------------------

    overriding
    procedure Remove_Event_Listeners( this : access Keen_Game ) is
    begin
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Player_Died" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Impulse" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Give_Points" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Give_Drops" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Give_Ammo" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Entity_Entered_Tile" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Dialog_Response" );
        Game(this.all).Remove_Event_Listeners;
    end Remove_Event_Listeners;

begin

    Register_Allocator( Create_Game'Access );

end Games.Keen;
