
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Palette_16;

package body Widgets.Buttons.Pushes.Menu_Buttons is

    BLINK_DELAY   : constant Time_Span := Milliseconds( 500 );

    ON_ICON       : constant String := "menu_dot_on.png";
    OFF_ICON      : constant String := "menu_dot_off.png";
    DISABLED_ICON : constant String := "menu_dot_disabled.png";

    ----------------------------------------------------------------------------

    function Create_Menu_Button( view : not null access Game_Views.Game_View'Class;
                                 id   : String;
                                 text : String := "";
                                 icon : String := "" ) return A_Button is
        this : A_Menu_Button := new Menu_Button;
    begin
        this.Construct( view, id, text, icon );
        return A_Button(this);
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Menu_Button;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Menu_Button;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String;
                         text : String;
                         icon : String ) is
    begin
        Push_Button(this.all).Construct( view, id, text, icon );
        this.border := Border_None;
        this.align := Align_Left;
        this.focusTime := Time_Span_Zero;
        this.Set_Color( Themes.Text, Palette_16.Green );
        this.Set_Color( Themes.Selected, Palette_16.Dark_Grey );
        this.Set_Color( Themes.Foreground, Palette_16.Dark_Grey );
        this.Set_Icon( OFF_ICON );
        this.Set_Shade_Disabled( 1.0 );
        this.Set_Font( "menu", 8 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Blur( this : access Menu_Button ) is
    begin
        Push_Button(this.all).Handle_Blur;
        this.Set_Color( Themes.Text, Palette_16.Green );
        this.Set_Icon( OFF_ICON );
    end Handle_Blur;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Disabled( this : access Menu_Button ) is
    begin
        this.Set_Icon( DISABLED_ICON );
    end Handle_Disabled;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Enabled( this : access Menu_Button ) is
    begin
        this.Set_Icon( OFF_ICON );
    end Handle_Enabled;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Focus( this : access Menu_Button ) is
    begin
        Push_Button(this.all).Handle_Focus;
        this.Set_Color( Themes.Text, Palette_16.Light_Green );
    end Handle_Focus;

    ----------------------------------------------------------------------------

    overriding
    function Handle_Key_Press( this : access Menu_Button;
                               evt  : not null A_Key_Event ) return Boolean is
    begin
        if evt.Get_Key = KEY_DOWN then
            if this.Focus_Next then
                return True;
            end if;
        elsif evt.Get_Key = KEY_UP then
            if this.Focus_Prev then
                return True;
            end if;
        elsif evt.Get_Key = KEY_TAB then
            -- prevent the default tab-focus-switching behavior
            return True;
        end if;
        return Push_Button(this.all).Handle_Key_Press( evt );
    end Handle_Key_Press;

    ----------------------------------------------------------------------------

    overriding
    procedure Tick( this : access Menu_Button; time : Tick_Time ) is
    begin
        if this.focused then
            if ((time.total - this.focusTime) / BLINK_DELAY) mod 2 = 0 then
                this.Set_Icon( ON_ICON );
            else
                this.Set_Icon( OFF_ICON );
            end if;
        else
            this.focusTime := time.total;
        end if;
    end Tick;

end Widgets.Buttons.Pushes.Menu_Buttons;
