/*
 * Copyright (C) 2003 Andrea Mazzoleni
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "pixel.h"
#include <stdlib.h>

extern void scale2x(unsigned char* dst_ptr,
		    unsigned dst_slice,
		    const unsigned char* src_ptr,
		    unsigned src_slice,
		    unsigned pixel,
		    unsigned width,
		    unsigned height,
		    int opt_tes)
{
    int x;
    int y;

    for(y=0;y<height;++y) {
        for(x=0;x<width;++x) {
            pixel_t E0, E1, E2, E3;
            pixel_t A, B, C, D, E, F, G, H, I;

            A = pixel_get(x-1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            B = pixel_get(x, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            C = pixel_get(x+1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            D = pixel_get(x-1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            E = pixel_get(x, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            F = pixel_get(x+1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            G = pixel_get(x-1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            H = pixel_get(x, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            I = pixel_get(x+1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);

            /*
                ABC
                DEF
                GHI

                E0E1
                E2E3
            */
            E0 = D == B && B != F && D != H ? D : E;
            E1 = B == F && B != D && F != H ? F : E;
            E2 = D == H && D != B && H != F ? D : E;
            E3 = H == F && D != H && B != F ? F : E;

            pixel_put(x*2, y*2, dst_ptr, dst_slice, pixel, width*2, height*2, E0);
            pixel_put(x*2+1, y*2, dst_ptr, dst_slice, pixel, width*2, height*2, E1);
            pixel_put(x*2, y*2+1, dst_ptr, dst_slice, pixel, width*2, height*2, E2);
            pixel_put(x*2+1, y*2+1, dst_ptr, dst_slice, pixel, width*2, height*2, E3);
        }
    }
}

extern void scale3x(unsigned char* dst_ptr,
		    unsigned dst_slice,
		    const unsigned char* src_ptr,
		    unsigned src_slice,
		    unsigned pixel,
		    unsigned width,
		    unsigned height,
		    int opt_tes)
{
    int x;
    int y;

    for(y=0;y<height;++y) {
        for(x=0;x<width;++x) {
            pixel_t E0, E1, E2, E3, E4, E5, E6, E7, E8;
            pixel_t A, B, C, D, E, F, G, H, I;
            int k0, k1, k2, k3;

            A = pixel_get(x-1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            B = pixel_get(x, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            C = pixel_get(x+1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            D = pixel_get(x-1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            E = pixel_get(x, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            F = pixel_get(x+1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            G = pixel_get(x-1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            H = pixel_get(x, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            I = pixel_get(x+1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);

            /*
                ABC
                DEF
                GHI

                k0k1
                k2k3

                E0E1E2
                E3E4E5
                E6E7E8
            */
            k0 = D == B && B != F && D != H;
            k1 = B == F && B != D && F != H;
            k2 = D == H && D != B && H != F;
            k3 = H == F && D != H && B != F;
            E0 = k0 ? D : E;
            E1 = (k0 && E != C) || (k1 && E != A) ? B : E;
            E2 = k1 ? F : E;
            E3 = (k0 && E != G) || (k2 && E != A) ? D : E;
            E4 = E;
            E5 = (k1 && E != I) || (k3 && E != C) ? F : E;
            E6 = k2 ? D : E;
            E7 = (k2 && E != I) || (k3 && E != G) ? H : E;
            E8 = k3 ? F : E;

            pixel_put(x*3, y*3, dst_ptr, dst_slice, pixel, width*3, height*3, E0);
            pixel_put(x*3+1, y*3, dst_ptr, dst_slice, pixel, width*3, height*3, E1);
            pixel_put(x*3+2, y*3, dst_ptr, dst_slice, pixel, width*3, height*3, E2);
            pixel_put(x*3, y*3+1, dst_ptr, dst_slice, pixel, width*3, height*3, E3);
            pixel_put(x*3+1, y*3+1, dst_ptr, dst_slice, pixel, width*3, height*3, E4);
            pixel_put(x*3+2, y*3+1, dst_ptr, dst_slice, pixel, width*3, height*3, E5);
            pixel_put(x*3, y*3+2, dst_ptr, dst_slice, pixel, width*3, height*3, E6);
            pixel_put(x*3+1, y*3+2, dst_ptr, dst_slice, pixel, width*3, height*3, E7);
            pixel_put(x*3+2, y*3+2, dst_ptr, dst_slice, pixel, width*3, height*3, E8);
        }
    }
}

extern int scale4x(unsigned char* dst_ptr,
		   unsigned dst_slice,
		   const unsigned char* src_ptr,
		   unsigned src_slice,
		   unsigned pixel,
		   unsigned width,
		   unsigned height,
		   int opt_tes)
{
    unsigned char* mid_ptr;
    unsigned mid_slice;

    mid_slice = width * pixel * 2;
    mid_ptr = malloc(mid_slice * height * 2);
    if (!mid_ptr) {
        return -1;
    }

    scale2x(mid_ptr, mid_slice, src_ptr, src_slice, pixel, width, height, opt_tes);
    scale2x(dst_ptr, dst_slice, mid_ptr, mid_slice, pixel, width * 2, height * 2, opt_tes);

    free(mid_ptr);

    return 0;
}

extern void scale2x3(unsigned char* dst_ptr,
		     unsigned dst_slice,
		     const unsigned char* src_ptr,
		     unsigned src_slice,
		     unsigned pixel,
		     unsigned width,
		     unsigned height,
		     int opt_tes)
{
    int x;
    int y;

    for(y=0;y<height;++y) {
        for(x=0;x<width;++x) {
            pixel_t E0, E1, E2, E3, E4, E5;
            pixel_t A, B, C, D, E, F, G, H, I;
            int k0, k1, k2, k3;

            A = pixel_get(x-1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            B = pixel_get(x, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            C = pixel_get(x+1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            D = pixel_get(x-1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            E = pixel_get(x, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            F = pixel_get(x+1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            G = pixel_get(x-1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            H = pixel_get(x, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            I = pixel_get(x+1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);

            /*
                ABC
                DEF
                GHI

                E0E1
                E2E3
                E4E5
            */
            k0 = D == B && B != F && D != H;
            k1 = B == F && B != D && F != H;
            k2 = D == H && D != B && H != F;
            k3 = H == F && D != H && B != F;
            E0 = k0 ? D : E;
            E1 = k1 ? F : E;
            E2 = (k0 && E != G) || (k2 && E != A) ? D : E;
            E3 = (k1 && E != I) || (k3 && E != C) ? F : E;
            E4 = k2 ? D : E;
            E5 = k3 ? F : E;

            pixel_put(x*2, y*3, dst_ptr, dst_slice, pixel, width*2, height*3, E0);
            pixel_put(x*2+1, y*3, dst_ptr, dst_slice, pixel, width*2, height*3, E1);
            pixel_put(x*2, y*3+1, dst_ptr, dst_slice, pixel, width*2, height*3, E2);
            pixel_put(x*2+1, y*3+1, dst_ptr, dst_slice, pixel, width*2, height*3, E3);
            pixel_put(x*2, y*3+2, dst_ptr, dst_slice, pixel, width*2, height*3, E4);
            pixel_put(x*2+1, y*3+2, dst_ptr, dst_slice, pixel, width*2, height*3, E5);
        }
    }
}

extern void scale2x4(unsigned char* dst_ptr,
		     unsigned dst_slice,
		     const unsigned char* src_ptr,
		     unsigned src_slice,
		     unsigned pixel,
		     unsigned width,
		     unsigned height,
		     int opt_tes)
{
    int x;
    int y;

    for(y=0;y<height;++y) {
        for(x=0;x<width;++x) {
            pixel_t E0, E1, E2, E3, E4, E5, E6, E7;
            pixel_t A, B, C, D, E, F, G, H, I;
            int k0, k1, k2, k3;

            A = pixel_get(x-1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            B = pixel_get(x, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            C = pixel_get(x+1, y-1, src_ptr, src_slice, pixel, width, height, opt_tes);
            D = pixel_get(x-1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            E = pixel_get(x, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            F = pixel_get(x+1, y, src_ptr, src_slice, pixel, width, height, opt_tes);
            G = pixel_get(x-1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            H = pixel_get(x, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);
            I = pixel_get(x+1, y+1, src_ptr, src_slice, pixel, width, height, opt_tes);

            /*
                ABC
                DEF
                GHI

                E0E1
                E2E3
                E4E5
                E6E7
            */
            k0 = D == B && B != F && D != H;
            k1 = B == F && B != D && F != H;
            k2 = D == H && D != B && H != F;
            k3 = H == F && D != H && B != F;
            E0 = k0 ? D : E;
            E1 = k1 ? F : E;
            E2 = (k0 && E != G) || (k2 && E != A) ? D : E;
            E3 = (k1 && E != I) || (k3 && E != C) ? F : E;
            E4 = (k0 && E != G) || (k2 && E != A) ? D : E;
            E5 = (k1 && E != I) || (k3 && E != C) ? F : E;
            E6 = k2 ? D : E;
            E7 = k3 ? F : E;

            pixel_put(x*2, y*4, dst_ptr, dst_slice, pixel, width*2, height*4, E0);
            pixel_put(x*2+1, y*4, dst_ptr, dst_slice, pixel, width*2, height*4, E1);
            pixel_put(x*2, y*4+1, dst_ptr, dst_slice, pixel, width*2, height*4, E2);
            pixel_put(x*2+1, y*4+1, dst_ptr, dst_slice, pixel, width*2, height*4, E3);
            pixel_put(x*2, y*4+2, dst_ptr, dst_slice, pixel, width*2, height*4, E4);
            pixel_put(x*2+1, y*4+2, dst_ptr, dst_slice, pixel, width*2, height*4, E5);
            pixel_put(x*2, y*4+3, dst_ptr, dst_slice, pixel, width*2, height*4, E6);
            pixel_put(x*2+1, y*4+3, dst_ptr, dst_slice, pixel, width*2, height*4, E7);
        }
    }
}
