
with Ada.Tags;                          use Ada.Tags;
with Entities;
with Entities.Enemies;                  use Entities.Enemies;
with Events.Audio;                      use Events.Audio;
with Events.Entities;                   use Events.Entities;
with Resources;                         use Resources;

pragma Elaborate_All( Entities );

package body Entities.Sprites.Keen4 is

    POINTS_LIFETIME : constant Time_Span := Milliseconds( 600 );

    ----------------------------------------------------------------------------

    function Create_Blue_1UP return A_Entity is
        this : A_Entity := new Blue_1UP;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Blue_1UP;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Blue_1UP ) is
    begin
        Sprite(this.all).Construct( 22, 12, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    yellow_1up_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Blue_1UP'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if yellow_1up_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "1up-blue.bmp" );
                Unload_Library( lib );
            end if;
            yellow_1up_frames := new Tile_Id_Array'(1 => id);
        end if;
        return yellow_1up_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Blue_1UP is
        this : aliased Blue_1UP;
    begin
        this.Construct;
        Blue_1UP'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Blue_1UP ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Blue_1UP ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Collect_Drop return A_Entity is
        this : A_Entity := new Collect_Drop;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Collect_Drop;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Collect_Drop ) is
    begin
        Sprite(this.all).Construct( 16, 16, "items", this.Frame_Array,
                                    Milliseconds( 315 ), Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    collect_drop_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Collect_Drop'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
    begin
        if collect_drop_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                collect_drop_frames := new Tile_Id_Array'(lib.Get_Id( "drop5.bmp" ),
                                                          lib.Get_Id( "drop6.bmp" ),
                                                          lib.Get_Id( "drop7.bmp" ));
                Unload_Library( lib );
            else
                collect_drop_frames := new Tile_Id_Array'(0, 0, 0);
            end if;
        end if;
        return collect_drop_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Collect_Drop is
        this : aliased Collect_Drop;
    begin
        this.Construct;
        Collect_Drop'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Collect_Drop ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Collect_Drop ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Collect_Gem return A_Entity is
        this : A_Entity := new Collect_Gem;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Collect_Gem;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Collect_Gem ) is
    begin
        Sprite(this.all).Construct( 15, 14, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    collect_gem_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Collect_Gem'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if collect_gem_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "gemgrab.bmp" );
                Unload_Library( lib );
            end if;
            collect_gem_frames := new Tile_Id_Array'(1 => id);
        end if;
        return collect_gem_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Collect_Gem is
        this : aliased Collect_Gem;
    begin
        this.Construct;
        Collect_Gem'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Collect_Gem ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Collect_Gem ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Collect_Stunner return A_Entity is
        this : A_Entity := new Collect_Stunner;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Collect_Stunner;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Collect_Stunner ) is
    begin
        Sprite(this.all).Construct( 13, 10, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    collect_stunner_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Collect_Stunner'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if collect_stunner_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "gungrab.bmp" );
                Unload_Library( lib );
            end if;
            collect_stunner_frames := new Tile_Id_Array'(1 => id);
        end if;
        return collect_stunner_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Collect_Stunner is
        this : aliased Collect_Stunner;
    begin
        this.Construct;
        Collect_Stunner'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Collect_Stunner ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Collect_Stunner ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Points_100 return A_Entity is
        this : A_Entity := new Points_100;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Points_100;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Points_100 ) is
    begin
        Sprite(this.all).Construct( 15, 8, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    points_100_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Points_100'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if points_100_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "100points.bmp" );
                Unload_Library( lib );
            end if;
            points_100_frames := new Tile_Id_Array'(1 => id);
        end if;
        return points_100_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Points_100 is
        this : aliased Points_100;
    begin
        this.Construct;
        Points_100'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Points_100 ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Points_100 ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Points_200 return A_Entity is
        this : A_Entity := new Points_200;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Points_200;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Points_200 ) is
    begin
        Sprite(this.all).Construct( 16, 8, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    points_200_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Points_200'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if points_200_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "200points.bmp" );
                Unload_Library( lib );
            end if;
            points_200_frames := new Tile_Id_Array'(1 => id);
        end if;
        return points_200_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Points_200 is
        this : aliased Points_200;
    begin
        this.Construct;
        Points_200'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Points_200 ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Points_200 ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Points_500 return A_Entity is
        this : A_Entity := new Points_500;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Points_500;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Points_500 ) is
    begin
        Sprite(this.all).Construct( 16, 8, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    points_500_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Points_500'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if points_500_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "500points.bmp" );
                Unload_Library( lib );
            end if;
            points_500_frames := new Tile_Id_Array'(1 => id);
        end if;
        return points_500_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Points_500 is
        this : aliased Points_500;
    begin
        this.Construct;
        Points_500'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Points_500 ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Points_500 ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Points_1000 return A_Entity is
        this : A_Entity := new Points_1000;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Points_1000;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Points_1000 ) is
    begin
        Sprite(this.all).Construct( 20, 8, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    points_1000_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Points_1000'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if points_1000_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "1000points.bmp" );
                Unload_Library( lib );
            end if;
            points_1000_frames := new Tile_Id_Array'(1 => id);
        end if;
        return points_1000_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Points_1000 is
        this : aliased Points_1000;
    begin
        this.Construct;
        Points_1000'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Points_1000 ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Points_1000 ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Points_2000 return A_Entity is
        this : A_Entity := new Points_2000;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Points_2000;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Points_2000 ) is
    begin
        Sprite(this.all).Construct( 21, 8, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    points_2000_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Points_2000'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if points_2000_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "2000points.bmp" );
                Unload_Library( lib );
            end if;
            points_2000_frames := new Tile_Id_Array'(1 => id);
        end if;
        return points_2000_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Points_2000 is
        this : aliased Points_2000;
    begin
        this.Construct;
        Points_2000'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Points_2000 ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Points_2000 ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Points_5000 return A_Entity is
        this : A_Entity := new Points_5000;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Points_5000;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Points_5000 ) is
    begin
        Sprite(this.all).Construct( 21, 8, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    points_5000_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Points_5000'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if points_5000_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "5000points.bmp" );
                Unload_Library( lib );
            end if;
            points_5000_frames := new Tile_Id_Array'(1 => id);
        end if;
        return points_5000_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Points_5000 is
        this : aliased Points_5000;
    begin
        this.Construct;
        Points_5000'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Points_5000 ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Points_5000 ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Stunner_Blast return A_Entity is
        this : A_Entity := new Stunner_Blast;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Stunner_Blast;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Stunner_Blast ) is
    begin
        Sprite(this.all).Construct( 16, 16, "sprites", this.Frame_Array,
                                    Milliseconds( 210 ), Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    stunner_blast_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Stunner_Blast'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
    begin
        if stunner_blast_frames = null then
            lib := Load_Library( "sprites" );
            stunner_blast_frames := new Tile_Id_Array'(1..2 => 0);
            if lib /= null then
                stunner_blast_frames(1) := lib.Get_Id( "phaze-blast1.bmp" );
                stunner_blast_frames(2) := lib.Get_Id( "phaze-blast2.bmp" );
                Unload_Library( lib );
            end if;
        end if;
        return stunner_blast_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Stunner_Blast is
        this : aliased Stunner_Blast;
    begin
        this.Construct;
        Stunner_Blast'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Stunner_Blast ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Stunner_Blast ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Stunner_Shot return A_Entity is
        this : A_Entity := new Stunner_Shot;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Stunner_Shot;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Stunner_Shot ) is
    begin
        Sprite(this.all).Construct( 12, 12, "sprites", this.Frame_Array,
                                    Time_Span_Zero, Milliseconds( 115 ) );
        this.physical := False;
        this.clipped := True;
    end Construct;

    ----------------------------------------------------------------------------

    procedure Explode( this : not null access Stunner_Shot'Class ) is
    begin
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Stunner_Blast'Tag ), this.x, this.y );
        Queue_Play_Sound( "shot_hit.wav" );
    end Explode;

    ----------------------------------------------------------------------------

    stunner_shot_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Stunner_Shot'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
    begin
        if stunner_shot_frames = null then
            lib := Load_Library( "sprites" );
            stunner_shot_frames := new Tile_Id_Array'(1..4 => 0);
            if lib /= null then
                stunner_shot_frames(1) := lib.Get_Id( "phaze1.bmp" );
                stunner_shot_frames(2) := lib.Get_Id( "phaze2.bmp" );
                stunner_shot_frames(3) := lib.Get_Id( "phaze3.bmp" );
                stunner_shot_frames(4) := lib.Get_Id( "phaze4.bmp" );
                Unload_Library( lib );
            end if;
        end if;
        return stunner_shot_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Stunner_Shot is
        this : aliased Stunner_Shot;
    begin
        this.Construct;
        Stunner_Shot'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Stunner_Shot ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Stunner_Shot ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Collide( this : access Stunner_Shot; e : not null A_Entity ) is
    begin
        if e.all in Enemy'Class then
            if A_Enemy(e).Is_Shootable then
                A_Enemy(e).Die;
                this.Explode;
            end if;
        end if;
    end On_Collide;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Hit_Wall( this : access Stunner_Shot; dir : Cardinal_Direction ) is
        pragma Unreferenced( dir );
    begin
        this.Explode;
    end On_Hit_Wall;

    --==========================================================================

    function Create_Yellow_1UP return A_Entity is
        this : A_Entity := new Yellow_1UP;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Yellow_1UP;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Yellow_1UP ) is
    begin
        Sprite(this.all).Construct( 22, 12, "items", this.Frame_Array,
                                    POINTS_LIFETIME, Time_Span_Zero );
        this.physical := False;
    end Construct;

    ----------------------------------------------------------------------------

    blue_1up_frames : A_Tile_Id_Array := null;

    function Frame_Array( this : not null access Yellow_1UP'Class ) return A_Tile_Id_Array is
        pragma Unreferenced( this );
        lib : A_Tile_Library;
        id  : Natural := 0;
    begin
        if blue_1up_frames = null then
            lib := Load_Library( "items" );
            if lib /= null then
                id := lib.Get_Id( "1up-yellow.bmp" );
                Unload_Library( lib );
            end if;
            blue_1up_frames := new Tile_Id_Array'(1 => id);
        end if;
        return blue_1up_frames;
    end Frame_Array;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Yellow_1UP is
        this : aliased Yellow_1UP;
    begin
        this.Construct;
        Yellow_1UP'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Yellow_1UP ) is
    begin
        Sprite'Read( stream, Sprite(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Yellow_1UP ) is
    begin
        Sprite'Write( stream, Sprite(obj) );
    end Object_Write;

begin

    Entities.Factory.Register_Class( External_Tag( Blue_1UP'Tag ),        Create_Blue_1UP'Access );
    Entities.Factory.Register_Class( External_Tag( Collect_Drop'Tag ),    Create_Collect_Drop'Access );
    Entities.Factory.Register_Class( External_Tag( Collect_Gem'Tag ),     Create_Collect_Gem'Access );
    Entities.Factory.Register_Class( External_Tag( Collect_Stunner'Tag ), Create_Collect_Stunner'Access );
    Entities.Factory.Register_Class( External_Tag( Points_100'Tag ),      Create_Points_100'Access );
    Entities.Factory.Register_Class( External_Tag( Points_200'Tag ),      Create_Points_200'Access );
    Entities.Factory.Register_Class( External_Tag( Points_500'Tag ),      Create_Points_500'Access );
    Entities.Factory.Register_Class( External_Tag( Points_1000'Tag ),     Create_Points_1000'Access );
    Entities.Factory.Register_Class( External_Tag( Points_2000'Tag ),     Create_Points_2000'Access );
    Entities.Factory.Register_Class( External_Tag( Points_5000'Tag ),     Create_Points_5000'Access );
    Entities.Factory.Register_Class( External_Tag( Stunner_Blast'Tag ),   Create_Stunner_Blast'Access );
    Entities.Factory.Register_Class( External_Tag( Stunner_Shot'Tag ),    Create_Stunner_Shot'Access );
    Entities.Factory.Register_Class( External_Tag( Yellow_1UP'Tag ),      Create_Yellow_1UP'Access );

end Entities.Sprites.Keen4;
