
with Events.Manager;

package body Events.Keen is

    function Create_Dialog_Desc( text : String ) return A_Dialog_Desc is
        this : constant A_Dialog_Desc := new Dialog_Desc;
    begin
        this.Construct( text );
        return this;
    end Create_Dialog_Desc;

    ----------------------------------------------------------------------------

    procedure Add_Choice( this : not null access Dialog_Desc'Class; text : String ) is
    begin
        this.choices.Append( text );
    end Add_Choice;

    ----------------------------------------------------------------------------

    overriding
    procedure Adjust( this : access Dialog_Desc ) is
    begin
        Object(this.all).Adjust;
        -- GNAT 6.4 warns: useless assignment of object to itself
--        this.text := this.text;
--        this.choices := this.choices;
    end Adjust;

    ----------------------------------------------------------------------------

     not overriding
     procedure Construct( this : access Dialog_Desc; text : String ) is
     begin
         Object(this.all).Construct;
         this.text := To_Unbounded_String( text );
     end Construct;

    ----------------------------------------------------------------------------

    function Get_Choices( this : not null access Dialog_Desc'Class ) return Natural is
    begin
        return Natural(this.choices.Length);
    end Get_Choices;

    ----------------------------------------------------------------------------

    function Get_Text( this : not null access Dialog_Desc'Class ) return String is
    begin
        return To_String( this.text );
    end Get_Text;

    ----------------------------------------------------------------------------

    function Get_Choice( this   : not null access Dialog_Desc'Class;
                         choice : Positive ) return String is
    begin
        if choice <= Natural(this.choices.Length) then
            return this.choices.Element( choice );
        else
            return "";
        end if;
    end Get_Choice;

    ----------------------------------------------------------------------------

    function Copy( src : A_Dialog_Desc ) return A_Dialog_Desc is
    begin
        return A_Dialog_Desc(Copy( A_Object(src) ));
    end Copy;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Dialog_Desc ) is
    begin
        Delete( A_Object(this) );
    end Delete;

    --==========================================================================

    overriding
    procedure Adjust( this : access Dialog_Event ) is
    begin
        Event(this.all).Adjust;
        this.desc := Copy( this.desc );
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this     : access Dialog_Event;
                         dialogId : Integer;
                         desc     : in out A_Dialog_Desc ) is
    begin
        Event(this.all).Construct( "Dialog" );
        this.dialogId := dialogId;
        this.desc := desc;
        desc := null;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Dialog_Event ) is
    begin
        Delete( this.desc );
        Event(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Get_Desc( this : not null access Dialog_Event'Class ) return A_Dialog_Desc is
    begin
        return this.desc;
    end Get_Desc;

    ----------------------------------------------------------------------------

    function Get_Dialog_Id( this : not null access Dialog_Event'Class ) return Integer is
    begin
        return this.dialogId;
    end Get_Dialog_Id;
    --==========================================================================

    not overriding
    procedure Construct( this     : access Dialog_Response_Event;
                         dialogId : Integer;
                         choice   : Positive ) is
    begin
        Event(this.all).Construct( "Dialog_Response" );
        this.dialogId := dialogId;
        this.choice := choice;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Choice( this : not null access Dialog_Response_Event'Class ) return Positive is
    begin
        return this.choice;
    end Get_Choice;

    ----------------------------------------------------------------------------

    function Get_Dialog_Id( this : not null access Dialog_Response_Event'Class ) return Integer is
    begin
        return this.dialogId;
    end Get_Dialog_Id;

    --==========================================================================

    not overriding
    procedure Construct( this : access Give_Ammo_Event; amount : Integer ) is
    begin
        Event(this.all).Construct( "Give_Ammo" );
        this.amount := amount;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Amount( this : not null access Give_Ammo_Event'Class ) return Integer is
    begin
        return this.amount;
    end Get_Amount;

    --==========================================================================

    not overriding
    procedure Construct( this : access Give_Drops_Event; amount : Integer ) is
    begin
        Event(this.all).Construct( "Give_Drops" );
        this.amount := amount;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Amount( this : not null access Give_Drops_Event'Class ) return Integer is
    begin
        return this.amount;
    end Get_Amount;

    --==========================================================================

    not overriding
    procedure Construct( this : access Give_Points_Event; amount : Integer ) is
    begin
        Event(this.all).Construct( "Give_Points" );
        this.amount := amount;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Amount( this : not null access Give_Points_Event'Class ) return Integer is
    begin
        return this.amount;
    end Get_Amount;

    --==========================================================================

    procedure Queue_Dialog( dialogId    : Integer;
                            description : in out A_Dialog_Desc ) is
        evt : A_Dialog_Event := new Dialog_Event;
    begin
        evt.Construct( dialogId, description );  -- consumes description
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Dialog;

    ----------------------------------------------------------------------------

    procedure Queue_Dialog_Response( dialogId : Integer; choice : Positive ) is
        evt : A_Dialog_Response_Event := new Dialog_Response_Event;
    begin
        evt.Construct( dialogId, choice );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Dialog_Response;

    ----------------------------------------------------------------------------

    procedure Queue_Give_Ammo( amount : Integer ) is
        evt : A_Give_Ammo_Event := new Give_Ammo_Event;
    begin
        evt.Construct( amount );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Give_Ammo;

    ----------------------------------------------------------------------------

    procedure Queue_Give_Drops( amount : Integer ) is
        evt : A_Give_Drops_Event := new Give_Drops_Event;
    begin
        evt.Construct( amount );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Give_Drops;

    ----------------------------------------------------------------------------

    procedure Queue_Give_Points( amount : Integer ) is
        evt : A_Give_Points_Event := new Give_Points_Event;
    begin
        evt.Construct( amount );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Give_Points;

end Events.Keen;
