
with Ada.Unchecked_Deallocation;

package Zip_Streams.Array_Streams is

    type A_SEA is access all Stream_Element_Array;

    procedure Delete is new Ada.Unchecked_Deallocation( Stream_Element_Array, A_SEA );

    ---------------------------------------------------------------------
    -- Array_Stream: stream based on an in-memory Stream_Element_Array --
    ---------------------------------------------------------------------
    type Array_Stream is new Root_Zipstream_Type with private;
    type A_Array_Stream is access all Array_Stream'Class;

    -- Set a value in the stream, the index will be set
    -- to null and old data in the stream will be lost.
    procedure Set( Str : in out Array_Stream; Buf : in out A_SEA );

    -- Close the Array_Stream and delete its backing buffer
    procedure Close( Str : in out Array_Stream );

    procedure Delete( Str : in out A_Array_Stream );

    Out_Of_Bounds,
    Null_Stream,
    Stream_Overflow : exception;

private

    -- Array Stream spec
    type Array_Stream is new Root_Zipstream_Type with
        record
            Buf  : A_SEA := null;
            Loc  : Stream_Element_Offset := 1;
            Name : Unbounded_String;
            ModificationTime : Time := Calendar.Convert(Ada.Calendar.Clock);
        end record;

    -- Read data from the stream.
    procedure Read( Stream : in out Array_Stream;
                    Item   : out Stream_Element_Array;
                    Last   : out Stream_Element_Offset );

    -- write data to the stream, starting from the current index.
    -- Data will be overwritten from index.
    procedure Write( Stream : in out Array_Stream;
                     Item   : Stream_Element_Array );

    -- Set the index on the stream
    procedure Set_Index( S : access Array_Stream; To : Positive );

    -- returns the index of the stream
    function Index( S : access Array_Stream ) return Integer;

    -- returns the Size of the stream
    function Size( S : access Array_Stream ) return Integer;

    -- sets the name of the stream
    procedure SetName( S : access Array_Stream; Name : String );

    -- returns the name of the stream
    function GetName( S : access Array_Stream ) return String;

    -- this procedure sets the ModificationTime of the stream
    procedure SetTime( S : access Array_Stream;
                       ModificationTime : Time );

    -- this procedure returns the ModificationTime of the stream
    function GetTime( S : access Array_Stream ) return Time;

    -- returns true if the index is at the end of the stream
    function End_Of_Stream( S : access Array_Stream ) return Boolean;

end Zip_Streams.Array_Streams;
